#!/usr/bin/env python3

import unittest
from todo_by import todo_by, todo_while

class TodoByTests(unittest.TestCase):
    def test_todo_by_future_date(self):
        @todo_by("2050-01-01")
        def my_function():
            self.assertTrue(True)  # Placeholder assertion

        # The function should execute without raising an error
        my_function()

    def test_todo_by_past_date(self):
        with self.assertRaises(RuntimeError) as cm:
            @todo_by("1970-01-01", "This is a comment")
            def my_function():
                self.assertTrue(True)  # Placeholder assertion

            # The function should raise a RuntimeError with the appropriate error message
            my_function()

        error_message = str(cm.exception)
        expected_error_message = "TODO by Jan 1, 1970 has passed: This is a comment"
        self.assertEqual(error_message, expected_error_message)

    def test_todo_by_no_comment(self):
        with self.assertRaises(RuntimeError) as cm:
            @todo_by("1970-01-01")
            def my_function():
                self.assertTrue(True)  # Placeholder assertion

            # The function should raise a RuntimeError without a comment in the error message
            my_function()

        error_message = str(cm.exception)
        expected_error_message = "TODO by Jan 1, 1970 has passed"
        self.assertEqual(error_message, expected_error_message)

    def test_todo_by_future_version(self):
        @todo_while("999.0.0", "setup.py")
        def my_function():
            self.assertTrue(True)  # Placeholder assertion

        # The function should execute without raising an error
        my_function()

    def test_todo_by_past_version_no_comment(self):

        import sys
        if sys.version_info >= (3, 8):
            from importlib import metadata
        else:
            import importlib_metadata as metadata

        def _get_version():
            try:
                version = metadata.version('todo_by')
            except:
                version = None
            return version

        with self.assertRaises(RuntimeError) as cm:
            @todo_while("1.0.0", "setup.py")
            def my_function():
                self.assertTrue(True)  # Placeholder assertion

            # The function should raise a RuntimeError with the appropriate error message
            my_function()

        error_message = str(cm.exception)
        expected_error_message = f"TODO version requirement '1.0.0' not satisfied by current v{_get_version()}"
        self.assertEqual(error_message, expected_error_message)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3

import re

def _get_version(file_path):
    with open(file_path, 'r') as file:
        content = file.read()
        pattern = r'\b(\d+\.\d+\.\d+(?:-[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*)?(?:\+[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*)?)\b'
        version_match = re.search(pattern, content)
        if version_match:
            return version_match.group(1)
    raise ValueError(f"Unable to extract version number from {file_path}")

def _compare_semver(version1, version2):
    def _split_version(version):
        pre_release = None
        build_metadata = None
        if '-' in version:
            version, pre_release = version.split('-', 1)
        if '+' in version:
            version, build_metadata = version.split('+', 1)
        return version, pre_release, build_metadata

    v1, pre1, build1 = _split_version(version1)
    v2, pre2, build2 = _split_version(version2)

    components1 = list(map(int, v1.split('.')))
    components2 = list(map(int, v2.split('.')))

    for i in range(max(len(components1), len(components2))):
        val1 = components1[i] if i < len(components1) else 0
        val2 = components2[i] if i < len(components2) else 0

        if val1 < val2:
            return -1
        elif val1 > val2:
            return 1

    if pre1 and not pre2:
        return -1
    elif not pre1 and pre2:
        return 1
    elif pre1 and pre2:
        if pre1 < pre2:
            return -1
        elif pre1 > pre2:
            return 1

    return 0



#!/usr/bin/env python3

import datetime
import functools
from .utils import _get_version, _compare_semver

def todo_by(date_str, comment=None):
    date = datetime.datetime.strptime(date_str, "%Y-%m-%d").date()
    now = datetime.date.today()

    if date < now:
        date_str = date.strftime("%b %-d, %Y")
        error_message = f"TODO by {date_str} has passed"
        if comment:
            error_message += f": {comment}"
        raise RuntimeError(error_message)

    def wrapper(f, date_str=date_str, comment=comment):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            return f(*args, **kwargs)
        return wrapped
    return functools.partial(wrapper)

def todo_while(version_str, file, comment=None):
    current_version = _get_version(file)
    comparison = _compare_semver(current_version, version_str)

    if comparison >= 0:
        error_message = f"TODO version requirement '{version_str}' not satisfied by current v{current_version}"
        if comment:
            error_message += f": {comment}"
        raise RuntimeError(error_message)

    def wrapper(f, version_str=version_str, comment=comment):
        @functools.wraps(f)
        def wrapped(*args, **kwargs):
            return f(*args, **kwargs)
        return wrapped
    return functools.partial(wrapper)



#!/usr/bin/env python3

from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='todo_by',
    version='1.2.0',
    author='Martin Simon',
    author_email='me@martinsimon.me',
    description=' Runtime lifetimes for comments.',
    url='https://github.com/barnumbirr/todo_by',
    download_url='https://github.com/barnumbirr/todo_by/archive/refs/heads/master.zip',
    packages=['todo_by'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)
